@@ -179,15 +179,14 @@ FREQUENCY-PENALTY, PRESENCE-PENALTY, SERVICE, STREAM, INFO see
                         "Called in (current-buffer)."
                         (when data ; if not data it is fail
                           (oai--debug "calbackmy %s %s %s" oai-timers--element-marker-variable-dict (current-buffer) data)
-                          (oai-block--insert-single-response end-marker data nil)
+                          (oai-block--insert-single-response end-marker data nil 'not-final)
                           (run-at-time 0 nil callback data))))
           (calbafin (lambda (data callback)
                       (ignore callback)
                       ;; (setq _callback _callback)
                       (when data ; if not data it is fail
                         (oai--debug "calbafin")
-                        (oai-block--insert-single-response end-marker data nil 'final)
-                        (oai-block--insert-single-response end-marker nil 'insertrole 'final) ; finalize
+                        (oai-block--insert-single-response end-marker data t)
                         (oai-timers--interrupt-current-request (oai-timers--get-keys-for-variable header-marker) #'oai-restapi--stop-tracking-url-request)))))
 
       (oai--debug "oai-prompt-request-chain2 %s %s %s %s" header-marker service model oai-timers-duration)
